@@ -5,43 +5,42 @@ import numpy as np
 import requests
 
 import pandas.compat as compat
-from pandas import Panel, DataFrame
-from pandas import read_csv
+from pandas import DataFrame
+from pandas import read_csv, concat
 from pandas.io.common import urlencode
 from pandas.compat import StringIO, bytes_to_str
 
 from pandas_datareader._utils import (RemoteDataError, SymbolWarning,
                                       _sanitize_dates, _init_session)
 
-import requests_ftp
-requests_ftp.monkeypatch_session()
-
 
 class _BaseReader(object):
-
     """
-
     Parameters
     ----------
-        sym : string with a single Single stock symbol (ticker).
-        start : string, (defaults to '1/1/2010')
-                Starting date, timestamp. Parses many different kind of date
-                representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
-        end : string, (defaults to today)
-                Ending date, timestamp. Same format as starting date.
-        retry_count : int, default 3
-                Number of times to retry query request.
-        pause : float, default 0.1
-                Time, in seconds, of the pause between retries.
-        session : Session, default None
-                requests.sessions.Session instance to be used
+    symbols : {str, List[str]}
+        String symbol of like of symbols
+    start : string, (defaults to '1/1/2010')
+        Starting date, timestamp. Parses many different kind of date
+        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
+    end : string, (defaults to today)
+        Ending date, timestamp. Same format as starting date.
+    retry_count : int, default 3
+        Number of times to retry query request.
+    pause : float, default 0.1
+        Time, in seconds, of the pause between retries.
+    session : Session, default None
+        requests.sessions.Session instance to be used
+    freq : {str, None}
+        Frequency to use in select readers
     """
 
     _chunk_size = 1024 * 1024
     _format = 'string'
 
-    def __init__(self, symbols, start=None, end=None,
-                 retry_count=3, pause=0.1, timeout=30, session=None):
+    def __init__(self, symbols, start=None, end=None,  retry_count=3,
+                 pause=0.1, timeout=30, session=None, freq=None):
+
         self.symbols = symbols
 
         start, end = _sanitize_dates(start, end)
@@ -53,20 +52,31 @@ class _BaseReader(object):
         self.retry_count = retry_count
         self.pause = pause
         self.timeout = timeout
+        self.pause_multiplier = 1
         self.session = _init_session(session, retry_count)
+        self.freq = freq
+
+    def close(self):
+        """Close network session"""
+        self.session.close()
 
     @property
     def url(self):
+        """API URL"""
         # must be overridden in subclass
         raise NotImplementedError
 
     @property
     def params(self):
+        """Parameters to use in API calls"""
         return None
 
     def read(self):
-        """ read data """
-        return self._read_one_data(self.url, self.params)
+        """Read data from connector"""
+        try:
+            return self._read_one_data(self.url, self.params)
+        finally:
+            self.close()
 
     def _read_one_data(self, url, params):
         """ read one data from specified URL """
@@ -85,6 +95,10 @@ class _BaseReader(object):
         response = self._get_response(url, params=params)
         text = self._sanitize_response(response)
         out = StringIO()
+        if len(text) == 0:
+            service = self.__class__.__name__
+            raise IOError("{} request returned no data; check URL for invalid "
+                          "inputs: {}".format(service, self.url))
         if isinstance(text, compat.binary_type):
             out.write(bytes_to_str(text))
         else:
@@ -99,7 +113,7 @@ class _BaseReader(object):
         """
         return response.content
 
-    def _get_response(self, url, params=None):
+    def _get_response(self, url, params=None, headers=None):
         """ send raw HTTP request to get requests.Response from the specified url
         Parameters
         ----------
@@ -110,17 +124,55 @@ class _BaseReader(object):
         """
 
         # initial attempt + retry
+        pause = self.pause
+        last_response_text = ''
         for i in range(self.retry_count + 1):
-            response = self.session.get(url, params=params)
+            response = self.session.get(url,
+                                        params=params,
+                                        headers=headers)
             if response.status_code == requests.codes.ok:
                 return response
-            time.sleep(self.pause)
+
+            last_response_text = response.text.encode(response.encoding)
+            time.sleep(pause)
+
+            # Increase time between subsequent requests, per subclass.
+            pause *= self.pause_multiplier
+            # Get a new breadcrumb if necessary, in case ours is invalidated
+            if isinstance(params, list) and 'crumb' in params:
+                params['crumb'] = self._get_crumb(self.retry_count)
+
+            # If our output error function returns True, exit the loop.
+            if self._output_error(response):
+                break
+
         if params is not None and len(params) > 0:
             url = url + "?" + urlencode(params)
-        raise RemoteDataError('Unable to read URL: {0}'.format(url))
+        msg = 'Unable to read URL: {0}'.format(url)
+        if last_response_text:
+            msg += '\nResponse Text:\n{0}'.format(last_response_text)
+
+        raise RemoteDataError(msg)
+
+    def _get_crumb(self, *args):
+        """ To be implemented by subclass """
+        raise NotImplementedError("Subclass has not implemented method.")
+
+    def _output_error(self, out):
+        """If necessary, a service can implement an interpreter for any non-200
+         HTTP responses.
+
+        :param out: raw output from an HTTP request
+        :return: boolean
+        """
+        return False
 
     def _read_lines(self, out):
-        rs = read_csv(out, index_col=0, parse_dates=True, na_values='-')[::-1]
+        rs = read_csv(out, index_col=0, parse_dates=True,
+                      na_values=('-', 'null'))[::-1]
+        # Needed to remove blank space character in header names
+        rs.columns = list(map(lambda x: x.strip(), rs.columns.values.tolist()))
+
         # Yahoo! Finance sometimes does this awesome thing where they
         # return 2 rows for the most recent business day
         if len(rs) > 2 and rs.index[-1] == rs.index[-2]:  # pragma: no cover
@@ -132,6 +184,7 @@ class _BaseReader(object):
         except AttributeError:
             # Python 3 string has no decode method.
             rs.index.name = rs.index.name.encode('ascii', 'ignore').decode()
+
         return rs
 
 
@@ -139,7 +192,7 @@ class _DailyBaseReader(_BaseReader):
     """ Base class for Google / Yahoo daily reader """
 
     def __init__(self, symbols=None, start=None, end=None, retry_count=3,
-                 pause=0.001, session=None, chunksize=25):
+                 pause=0.1, session=None, chunksize=25):
         super(_DailyBaseReader, self).__init__(symbols=symbols,
                                                start=start, end=end,
                                                retry_count=retry_count,
@@ -150,7 +203,7 @@ class _DailyBaseReader(_BaseReader):
         raise NotImplementedError
 
     def read(self):
-        """ read data """
+        """Read data"""
         # If a single symbol, (e.g., 'GOOG')
         if isinstance(self.symbols, (compat.string_types, int)):
             df = self._read_one_data(self.url,
@@ -186,7 +239,9 @@ class _DailyBaseReader(_BaseReader):
                 df_na[:] = np.nan
                 for sym in failed:
                     stocks[sym] = df_na
-            return Panel(stocks).swapaxes('items', 'minor')
+            result = concat(stocks).unstack(level=0)
+            result.columns.names = ['Attributes', 'Symbols']
+            return result
         except AttributeError:
             # cannot construct a panel with just 1D nans indicating no data
             msg = "No data fetched using {0!r}"
