@@ -4,16 +4,17 @@ import requests
 import numpy as np
 import pandas as pd
 from pandas import DataFrame
-
+from requests.exceptions import ConnectionError
 import pytest
 import pandas.util.testing as tm
 
 import pandas_datareader.data as web
 from pandas_datareader.data import YahooDailyReader
-from pandas_datareader.yahoo.quotes import _yahoo_codes
 from pandas_datareader._utils import RemoteDataError
 from pandas_datareader._testing import skip_on_exception
 
+XFAIL_REASON = 'Known connection failures on Yahoo when testing!'
+
 
 class TestYahoo(object):
 
@@ -25,9 +26,10 @@ class TestYahoo(object):
     def test_yahoo(self):
         # Asserts that yahoo is minimally working
         start = datetime(2010, 1, 1)
-        end = datetime(2013, 1, 27)
+        end = datetime(2013, 1, 25)
 
-        assert web.DataReader('F', 'yahoo', start, end)['Close'][-1] == 13.68
+        assert round(web.DataReader('F', 'yahoo', start, end)['Close'][-1],
+                     2) == 13.68
 
     def test_yahoo_fails(self):
         start = datetime(2010, 1, 1)
@@ -37,23 +39,38 @@ class TestYahoo(object):
             web.DataReader('NON EXISTENT TICKER', 'yahoo', start, end)
 
     def test_get_quote_series(self):
-        df = web.get_quote_yahoo(pd.Series(['GOOG', 'AAPL', 'GOOG']))
-        tm.assert_series_equal(df.iloc[0], df.iloc[2])
+        stringlist = ['GOOG', 'AAPL']
+        fields = ['exchange', 'sharesOutstanding', 'epsForward']
+        try:
+            AAPL = web.get_quote_yahoo('AAPL')
+            df = web.get_quote_yahoo(pd.Series(stringlist))
+        except ConnectionError:
+            pytest.xfail(reason=XFAIL_REASON)
+        tm.assert_series_equal(AAPL.iloc[0][fields], df.loc['AAPL'][fields])
+        assert sorted(stringlist) == sorted(list(df.index.values))
 
     def test_get_quote_string(self):
-        _yahoo_codes.update({'MarketCap': 'j1'})
-        df = web.get_quote_yahoo('GOOG')
-        assert not pd.isnull(df['MarketCap'][0])
+        try:
+            df = web.get_quote_yahoo('GOOG')
+        except ConnectionError:
+            pytest.xfail(reason=XFAIL_REASON)
+
+        assert not pd.isnull(df['marketCap'][0])
 
     def test_get_quote_stringlist(self):
-        df = web.get_quote_yahoo(['GOOG', 'AAPL', 'GOOG'])
-        tm.assert_series_equal(df.iloc[0], df.iloc[2])
+        stringlist = ['GOOG', 'AAPL']
+        try:
+            df = web.get_quote_yahoo(stringlist)
+        except ConnectionError:
+            pytest.xfail(reason=XFAIL_REASON)
+        assert sorted(stringlist) == sorted(list(df.index.values))
 
     def test_get_quote_comma_name(self):
-        _yahoo_codes.update({'name': 'n'})
-        df = web.get_quote_yahoo(['RGLD'])
-        del _yahoo_codes['name']
-        assert df['name'][0] == 'Royal Gold, Inc.'
+        try:
+            df = web.get_quote_yahoo(['RGLD'])
+        except ConnectionError:
+            pytest.xfail(reason=XFAIL_REASON)
+        assert df['longName'][0] == 'Royal Gold, Inc.'
 
     @pytest.mark.skip('Unreliable test, receive partial '
                       'components back for dow_jones')
@@ -97,6 +114,10 @@ class TestYahoo(object):
         # just test that we succeed
         web.get_data_yahoo('GOOG')
 
+    @skip_on_exception(RemoteDataError)
+    def test_data_with_no_actions(self):
+        web.get_data_yahoo('TSLA')
+
     @skip_on_exception(RemoteDataError)
     def test_get_data_adjust_price(self):
         goog = web.get_data_yahoo('GOOG')
@@ -147,23 +168,22 @@ class TestYahoo(object):
 
     @skip_on_exception(RemoteDataError)
     def test_get_data_multiple_symbols_two_dates(self):
-        pan = web.get_data_yahoo(['GE', 'MSFT', 'INTC'], 'JAN-01-12',
-                                 'JAN-31-12')
-        result = pan.Close['01-18-12'].T
+        data = web.get_data_yahoo(['GE', 'MSFT', 'INTC'], 'JAN-01-12',
+                                  'JAN-31-12')
+        result = data.Close.loc['01-18-12'].T
         assert result.size == 3
 
         # sanity checking
-        assert result.dtypes.all() == np.floating
+        assert result.dtypes == np.floating
 
         expected = np.array([[18.99, 28.4, 25.18],
                              [18.58, 28.31, 25.13],
                              [19.03, 28.16, 25.52],
                              [18.81, 28.82, 25.87]])
-        df = pan.Open
+        df = data.Open
         result = df[(df.index >= 'Jan-15-12') & (df.index <= 'Jan-20-12')]
         assert expected.shape == result.shape
 
-    @pytest.mark.xfail(reason="failing after #355")
     def test_get_date_ret_index(self):
         pan = web.get_data_yahoo(['GE', 'INTC', 'IBM'], '1977', '1987',
                                  ret_index=True)
@@ -177,21 +197,21 @@ class TestYahoo(object):
         # sanity checking
         assert np.issubdtype(pan.values.dtype, np.floating)
 
-    @pytest.mark.xfail(reason="failing after #355")
     def test_get_data_yahoo_actions(self):
         start = datetime(1990, 1, 1)
         end = datetime(2000, 4, 5)
 
-        actions = web.get_data_yahoo_actions('BHP.AX', start, end)
+        actions = web.get_data_yahoo_actions('BHP.AX', start, end,
+                                             adjust_dividends=True)
 
-        assert sum(actions['action'] == 'DIVIDEND') == 20
+        assert sum(actions['action'] == 'DIVIDEND') == 21
         assert sum(actions['action'] == 'SPLIT') == 1
 
         assert actions.loc['1995-05-11', 'action'][0] == 'SPLIT'
         assert actions.loc['1995-05-11', 'value'][0] == 1 / 1.1
 
         assert actions.loc['1993-05-10', 'action'][0] == 'DIVIDEND'
-        assert actions.loc['1993-05-10', 'value'][0] == 0.3
+        assert actions.loc['1993-05-10', 'value'][0] == 0.21
 
     def test_get_data_yahoo_actions_invalid_symbol(self):
         start = datetime(1990, 1, 1)
@@ -205,14 +225,13 @@ class TestYahoo(object):
         r = YahooDailyReader('GOOG')
         df = r.read()
 
-        assert df.Volume.loc['JAN-02-2015'] == 1447500
+        assert df.Volume.loc['JAN-02-2015'] == 1447600
 
         session = requests.Session()
 
         r = YahooDailyReader('GOOG', session=session)
         assert r.session is session
 
-    @pytest.mark.xfail(reason="failing after #355")
     def test_yahoo_DataReader(self):
         start = datetime(2010, 1, 1)
         end = datetime(2015, 5, 9)
@@ -225,6 +244,22 @@ class TestYahoo(object):
                                     '2013-08-08', '2013-05-09',
                                     '2013-02-07', '2012-11-07',
                                     '2012-08-09'])
+
+        exp = pd.DataFrame({'action': ['DIVIDEND', 'DIVIDEND', 'DIVIDEND',
+                                       'DIVIDEND', 'SPLIT', 'DIVIDEND',
+                                       'DIVIDEND', 'DIVIDEND',
+                                       'DIVIDEND', 'DIVIDEND', 'DIVIDEND',
+                                       'DIVIDEND', 'DIVIDEND'],
+                            'value': [0.52, 0.47, 0.47, 0.47, 0.14285714,
+                                      3.29, 3.05, 3.05, 3.05,
+                                      3.05, 2.65, 2.65, 2.65]},
+                           index=exp_idx)
+        exp.index.name = 'Date'
+        tm.assert_frame_equal(result.reindex_like(exp).round(2), exp.round(2))
+
+        result = web.get_data_yahoo_actions('AAPL', start, end,
+                                            adjust_dividends=True)
+
         exp = pd.DataFrame({'action': ['DIVIDEND', 'DIVIDEND', 'DIVIDEND',
                                        'DIVIDEND', 'SPLIT', 'DIVIDEND',
                                        'DIVIDEND', 'DIVIDEND',
@@ -235,13 +270,11 @@ class TestYahoo(object):
                                       0.43571, 0.37857, 0.37857, 0.37857]},
                            index=exp_idx)
         exp.index.name = 'Date'
-
-        tm.assert_frame_equal(result.reindex_like(exp), exp)
+        tm.assert_frame_equal(result.reindex_like(exp).round(5), exp.round(5))
 
     @skip_on_exception(RemoteDataError)
     def test_yahoo_DataReader_multi(self):
         start = datetime(2010, 1, 1)
         end = datetime(2015, 5, 9)
-
         result = web.DataReader(['AAPL', 'F'], 'yahoo-actions', start, end)
-        assert isinstance(result, pd.Panel)
+        assert isinstance(result, dict)
